import os
import platform
import sys

from winreg import *


class MariaDBConfiguration():
    lib_dirs = []
    libs = []
    version = []
    includes = []
    extra_objects = []
    extra_compile_args= []
    extra_link_args= []


def get_config(options):
    static= options["link_static"];
    mariadb_dir= options["install_dir"]
    required_version= "3.1.5"

    if not os.path.exists(mariadb_dir):
      try:
          mariadb_dir= os.environ["MARIADB_CC_INSTALL_DIR"]
          cc_version = ["", ""]
          cc_instdir = [mariadb_dir, ""]
          print("using environment configuration " + mariadb_dir)
      except KeyError:

          try:
              local_reg = ConnectRegistry(None, HKEY_LOCAL_MACHINE)
              if platform.architecture()[0] == '32bit':
                  connector_key = OpenKey(local_reg,
                                          'SOFTWARE\\MariaDB Corporation\\MariaDB Connector C')
              else:
                  connector_key = OpenKey(local_reg,
                                          'SOFTWARE\\MariaDB Corporation\\MariaDB Connector C 64-bit',
                                          access=KEY_READ | KEY_WOW64_64KEY)
              cc_version = QueryValueEx(connector_key, "Version")
              if cc_version[0] < required_version:
                  print("MariaDB Connector/Python requires MariaDB Connector/C >= %s (found version: %s") \
                       % (required_version, cc_version[0])
                  sys.exit(2)
              mariadb_dir = QueryValueEx(connector_key, "InstallDir")[0]

          except:
              print("Could not find InstallationDir of MariaDB Connector/C. "
                    "Please make sure MariaDB Connector/C is installed or specify the InstallationDir of "
                    "MariaDB Connector/C by setting the environment variable MARIADB_CC_INSTALL_DIR.")
              sys.exit(3)

    print("Found MariaDB Connector/C in '%s'" % mariadb_dir)
    cfg = MariaDBConfiguration()
    cfg.includes = [".\\include", mariadb_dir + "\\include", mariadb_dir + "\\include\\mysql"]
    cfg.lib_dirs = [mariadb_dir + "\\lib"]
    cfg.libs = ["ws2_32", "advapi32", "kernel32", "shlwapi", "crypt32"]
    if static.lower() == "on":
        cfg.libs.append("mariadbclient")
    else:
        cfg.libs.append("libmariadb")
    cfg.extra_link_args= ["/NODEFAULTLIB:LIBCMT"]
    cfg.extra_compile_args=["\"-DDEFAULT_PLUGINS_SUBDIR=\\\"\"" + options["install_dir"].replace(""'\\', '/') + "/lib/plugin\"\\\"\"", "/MD"]
    return cfg



#!/usr/bin/env python -O
# -*- coding: utf-8 -*-

import collections
import datetime
import unittest

import mariadb
import platform

from test.base_test import create_connection, conf


@unittest.skipIf(platform.python_implementation() == "PyPy", "skip pooling tests for PyPy")
class TestPooling(unittest.TestCase):

    def setUp(self):
        self.connection = create_connection()
        self.connection.autocommit = False

    def tearDown(self):
        del self.connection

    def test_connection_pools(self):
        pool= mariadb.ConnectionPool(pool_name="test")
        self.assertEqual(mariadb._CONNECTION_POOLS["test"], pool)
        del pool
        self.assertEqual(mariadb._CONNECTION_POOLS, {})

    def test_connection_pool_conf(self):
        pool= mariadb.ConnectionPool(pool_name="test")
        default_conf= conf()
        conn= create_connection()
        try:
            pool.add_connection(conn)
        except mariadb.PoolError:
            pass
        pool.set_config(**default_conf)
        pool.add_connection(conn)
        c= pool.get_connection()
        self.assertEqual(c, conn)
        del pool
    
    def test_connection_pool_maxconn(self):
        default_conf= conf()
        pool= mariadb.ConnectionPool(pool_name="test1", **default_conf)
        connections= []
        for i in range(1, 6):
           connections.append(pool.get_connection())
        try:
            x= pool.get_connection()
            print("ok")
        except mariadb.PoolError:
            pass
        for c in connections:
            c.close()
        x= pool.get_connection()
        print("ok")
        del pool

    def test_connection_pool_add(self):
        default_conf= conf()
        pool= mariadb.ConnectionPool(pool_name="test1")
        pool.set_config(**default_conf)
        for i in range(1,6):
            pool.add_connection()
        try:
            pool.add_connection()
        except mariadb.PoolError:
            pass
        del pool

    def test__CONNECTION_POOLS(self):
        default_conf= conf()
        pool= mariadb.ConnectionPool(pool_name="test", **default_conf)
        conn= mariadb.connect(pool_name="test")
        cursor= conn.cursor()
        cursor.execute("SELECT 1")
        row= cursor.fetchone()
        self.assertEqual(row[0], 1)
        del cursor
        del pool

    def test_create_pool_from_conn(self):
        default_conf= conf()
        key= "t1"
        conn= mariadb.connect(pool_name=key, **default_conf)
        cursor=conn.cursor()
        del mariadb._CONNECTION_POOLS["t1"]
        self.assertEqual(mariadb._CONNECTION_POOLS, {})
        try:
            cursor.execute("SELECT 1")
        except mariadb.ProgrammingError:
            pass

    def test_pool_getter(self):
        default_conf= conf()
        conn= mariadb.connect(pool_name="getter_test", pool_size=4, **default_conf)
        p= mariadb._CONNECTION_POOLS["getter_test"]
        self.assertEqual(p.pool_name, "getter_test")
        self.assertEqual(p.pool_size, 4)
        if "pool_reset_connection" in default_conf:
            self.assertEqual(p.pool_reset_connection, default_conf["pool_reset_connection"])
        else:
            self.assertEqual(p.pool_reset_connection, True)
        self.assertEqual(p.max_size, 64)
        del mariadb._CONNECTION_POOLS["getter_test"]

    def test_pool_connection_reset(self):
        default_conf= conf()
        conn= mariadb.connect(pool_name="reset_test", pool_size=1, **default_conf)
        cursor= conn.cursor()
        cursor.execute("SELECT 1")
        cursor.close()
        conn.close()
        conn= mariadb.connect(pool_name="reset_test")
        cursor= conn.cursor()
        cursor.execute("SELECT 2")
        row= cursor.fetchone()
        self.assertEqual(row[0], 2)
        del mariadb._CONNECTION_POOLS["reset_test"]

    def test_pool_add(self):
        default_conf= conf()
        pool= mariadb.ConnectionPool(pool_name="test")
        pool1= mariadb.ConnectionPool(pool_name="test1")
        pool.set_config(**default_conf)
        pool1.set_config(**default_conf)
        conn= create_connection()
        pool.add_connection(conn)
        try:
           pool.add_connection(conn)
        except mariadb.PoolError as e:
           pass
        try:
           pool1.add_connection(conn)
        except mariadb.PoolError as e:
           pass
        del pool, pool1

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

import os

from setuptools import setup, Extension
from configparser import ConfigParser

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# read settings from site.cfg 
c= ConfigParser()
c.read(['site.cfg'])
options= dict(c.items('cc_options'))

if os.name == "posix":
    from mariadb_posix import get_config
if os.name == "nt":
    from mariadb_windows import get_config

cfg = get_config(options)



setup(name='mariadb',
      version='0.9.52',
      python_requires='>=3.6',
      classifiers = [
          'Development Status :: 3 - Alpha',
          'Environment :: Console',
          'Environment :: MacOS X',
          'Environment :: Win32 (MS Windows)',
          'License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)',
          'Programming Language :: C',
          'Programming Language :: Python',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
          'Programming Language :: Python :: 3.8',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: MacOS',
          'Operating System :: POSIX',
          'Intended Audience :: End Users/Desktop',
          'Intended Audience :: Developers',
          'Intended Audience :: System Administrators',
          'Topic :: Database'
      ],
      description='Python MariaDB extension',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Georg Richter',
      license='LGPL 2.1',
      url='https://www.github.com/mariadb-corporation/mariadb-connector-python',
      project_urls={
         "Bug Tracker": "https://jira.mariadb.org/",
         "Documentation": "https://github.com/mariadb-corporation/mariadb-connector-python/wiki",
         "Source Code": "https://www.github.com/mariadb-corporation/mariadb-connector-python",
      },
      ext_modules=[Extension('mariadb', ['src/mariadb.c', 'src/mariadb_connection.c',
                                         'src/mariadb_exception.c', 'src/mariadb_cursor.c',
                                         'src/mariadb_codecs.c', 'src/mariadb_field.c',
                                         'src/mariadb_parser.c',
                                         'src/mariadb_pooling.c',
                                         'src/mariadb_dbapitype.c', 'src/mariadb_indicator.c'],
                             include_dirs=cfg.includes,
                             library_dirs=cfg.lib_dirs,
                             libraries=cfg.libs,
                             extra_compile_args = cfg.extra_compile_args,
                             extra_link_args = cfg.extra_link_args,
                             extra_objects= cfg.extra_objects
                             )],
      )



#!/usr/bin/env python -O
# -*- coding: utf-8 -*-

import datetime
import unittest

from test.base_test import create_connection


class CursorMariaDBTest(unittest.TestCase):

    def setUp(self):
        self.connection = create_connection()

    def tearDown(self):
        del self.connection

        def test_insert_parameter(self):
            cursor = self.connection.cursor()
            cursor.execute(
                "CREATE TEMPORARY TABLE test_insert_parameter(a int not null auto_increment primary key, b int, c int, d varchar(20),e date)")
            #    cursor.execute("set @@autocommit=0");
            list_in = []
            for i in range(1, 300001):
                row = (i, i, i, "bar", datetime.date(2019, 1, 1))
                list_in.append(row)
            cursor.executemany("INSERT INTO test_insert_parameter VALUES (?,?,?,?,?)", list_in)
            self.assertEqual(len(list_in), cursor.rowcount)
            self.connection.commit()
            cursor.execute("SELECT * FROM test_insert_parameter order by a")
            list_out = cursor.fetchall()
            self.assertEqual(len(list_in), cursor.rowcount);
            self.assertEqual(list_in, list_out)
            cursor.close()

        def test_update_parameter(self):
            cursor = self.connection.cursor()
            cursor.execute(
                "CREATE TEMPORARY TABLE test_update_parameter(a int not null auto_increment "
                "primary key, b int, c int, d varchar(20),e date)")
            cursor.execute("set @@autocommit=0");
            list_in = []
            for i in range(1, 300001):
                row = (i, i, i, "bar", datetime.date(2019, 1, 1))
                list_in.append(row)
            cursor.executemany("INSERT INTO test_update_parameter VALUES (?,?,?,?,?)", list_in)
            self.assertEqual(len(list_in), cursor.rowcount)
            self.connection.commit()
            cursor.close()
            list_update = [];

            cursor = self.connection.cursor()
            cursor.execute("set @@autocommit=0");
            for i in range(1, 300001):
                row = (i + 1, i);
                list_update.append(row);

            cursor.executemany("UPDATE test_update_parameter SET b=? WHERE a=?", list_update);
            self.assertEqual(cursor.rowcount, 300000)
            self.connection.commit();
            cursor.close()

        def test_delete_parameter(self):
            cursor = self.connection.cursor()
            cursor.execute(
                "CREATE TEMPORARY TABLE test_delete_parameter(a int not null auto_increment "
                "primary key, b int, c int, d varchar(20),e date)")
            cursor.execute("set @@autocommit=0");
            list_in = []
            for i in range(1, 300001):
                row = (i, i, i, "bar", datetime.date(2019, 1, 1))
                list_in.append(row)
            cursor.executemany("INSERT INTO test_delete_parameter VALUES (?,?,?,?,?)", list_in)
            self.assertEqual(len(list_in), cursor.rowcount)
            self.connection.commit()
            cursor.close()
            list_delete = [];

            cursor = self.connection.cursor()
            cursor.execute("set @@autocommit=0");
            for i in range(1, 300001):
                list_delete.append((i,));

            cursor.executemany("DELETE FROM test_delete_parameter WHERE a=?", list_delete);
            self.assertEqual(cursor.rowcount, 300000)
            self.connection.commit();
            cursor.close()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python -O

import mariadb
import datetime
import unittest

class CursorTest(unittest.TestCase):

  iterations= 500000

  def setUp(self):
    self.connection= mariadb.connection(default_file='default.cnf')

  def tearDown(self):
    self.connection.close()
    del self.connection

  def test_insert_parameter(self):
    cursor= self.connection.cursor()
    cursor.execute("DROP TABLE IF EXISTS t1")
    cursor.execute("CREATE OR REPLACE TABLE t1(a int not null auto_increment primary key, b int, c int, d varchar(20),e date)")
    cursor.execute("set @@autocommit=0");
    for i in range(1, self.iterations + 1):
      cursor.execute("INSERT INTO t1 VALUES (?,?,?,?,?)", (i,i,i, "bar", datetime.date(2019,1,1)))
    self.connection.commit()
    cursor.execute("SELECT * FROM t1 order by a")
    self.assertEqual(self.iterations, cursor.rowcount);
    cursor.execute("drop table t1")
    cursor.close()
